package org.geekfu.OldCartographer;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.MouseListener;
import java.io.Serializable;
import java.util.HashMap;

import javax.swing.JComponent;

public class Map extends JComponent implements Cloneable,Serializable{
	protected int width,height;
	protected MapCell cells[][];
	protected CellComponent components[][];
	protected Dimension cellsize;
	protected MapLayout layout;

	protected java.util.Map<MapCell,CellComponent> pieces=new HashMap<MapCell,CellComponent>();

	public Map() {
		width=height=0;
		cells=null;
		components=null;
	}

	public Map(int width,int height) {
		this(width,height,new MapCell(),new Dimension(32,32));
	}

	public Map(int width,int height,Dimension cellsize) {
		this(width,height,new MapCell(),cellsize);
	}

	public Map(int width,int height,MapCell cell,Dimension cellsize) {
		initMembers(width, height, cellsize);

		layoutComponent(cell);
	}

	protected void initMembers(int width, int height, Dimension cellsize) {
		this.width=width;
		this.height=height;
		this.cellsize=cellsize;
		cells=new MapCell[width][height];
		components=new CellComponent[width][height];
	}

	protected void layoutComponent(MapCell cell) {
		double[] col=new double[width];
		double[] row=new double[height];

		for(int n=0;n!=width;n++)
			col[n]=cellsize.width;

		for(int n=0;n!=height;n++)
			row[n]=cellsize.height;

		setLayout(layout=new MapLayout(this));

		cell.setSize(cellsize); // How large the cells are supposed to be

		for(int y=0;y!=height;y++)
			for(int x=0;x!=width;x++){
				cells[x][y]=(MapCell) cell.clone();
				components[x][y]=new CellComponent(this, cells[x][y],x,y);
				add(components[x][y],new MapLayoutConstraints(x,y));
			}
	}

	protected MapCell[][] getCells(){
		return cells;
	}
	public MapCell getSpace(int x,int y) {
		if(!inBounds(x, y))return null;
		return cells[x][y];
	}
	public MapCell getSpace(Point p) {
		return getSpace(p.x,p.y);
	}
	public CellComponent getComponent(Point p) {
		return getComponent(p.x,p.y);
	}
	public CellComponent getComponent(int x,int y) {
		if(!inBounds(x, y))return null;
		return components[x][y];
	}
	public void putSpace(Point p,MapCell cell) {
		putSpace(p.x,p.y,cell);
	}

	public void putSpace(int x,int y,MapCell cell) {
		if(!inBounds(x, y))return;
		cells[x][y]=(MapCell) cell.clone();
		cells[x][y].setSize(cellsize);
		components[x][y].setCell(cells[x][y]);
		validate();
	}

	public boolean inBounds(int x, int y) {
		return !(x<0 || y<0 || x>=getMapWidth() || y>=getMapHeight());
	}

	public boolean inBounds(Point p) {
		return inBounds(p.x,p.y);
	}

	public int getMapHeight() {
		return height;
	}

	public int getMapWidth() {
		return width;
	}

	public void setMapHeight(int height) {
		this.height = height;
	}

	public void setMapWidth(int width) {
		this.width = width;
	}

	public Dimension getCellsize() {
		return cellsize;
	}

	public void setCellsize(Dimension cellsize) {
		this.cellsize = cellsize;
		setMaximumSize(new Dimension(width*cellsize.width,height*cellsize.height));
		for(int x=0;x!=width;x++)
			for(int y=0;y!=height;y++)
			{
				CellComponent c=components[x][y];
				c.setPreferredSize(cellsize);
				MapCell m=cells[x][y];
				m.setSize(cellsize);
			}

		for(MapCell m : pieces.keySet()){
			m.setSize(cellsize);
		}

		this.revalidate();
	}

	public void repaintComponent(int x, int y) {
		if(!inBounds(x,y))return;
		components[x][y].repaint();
	}

	public synchronized void addMouseListener(MouseListener arg0) {
		for(int y=0;y!=getMapHeight();y++)
			for(int x=0;x!=getMapWidth();x++)
				components[x][y].addMouseListener(arg0);

		for(CellComponent c : pieces.values())
			c.addMouseListener(arg0);
		
		super.addMouseListener(arg0);
	}

	public synchronized void removeMouseListener(MouseListener arg0) {
		for(int y=0;y!=getMapHeight();y++)
			for(int x=0;x!=getMapWidth();x++)
				components[x][y].removeMouseListener(arg0);

		for(CellComponent c : pieces.values())
			c.removeMouseListener(arg0);

		super.removeMouseListener(arg0);
	}

	public CellComponent putPiece(int x,int y,MapCell piece) {
		return putPiece(x,y,0,0,piece);
	}

	public CellComponent putPiece(int x,int y,int xOff,int yOff,MapCell piece) {
		MapLayoutConstraints constraints=new MapLayoutConstraints(x,y,xOff,yOff);
		CellComponent component=null;
		if(pieces.containsKey(piece)){
			component=pieces.get(piece);
			remove(component);
			add(component,constraints,0);
			component.setPosition();
			component.repaint();
		}else{
			piece.setSize(cellsize);
			component=new CellComponent(this,piece,x,y);
			add(component,constraints,0);
			component.setPosition();
			pieces.put(piece, component);
			
			for(MouseListener l : getMouseListeners())
				component.addMouseListener(l);
		}
		piece.setConstraints(constraints);
		revalidate();
		return component;
	}

	public CellComponent putPiece(Point p,MapCell piece) {
		return putPiece(p.x,p.y,piece);
	}

	private void removePiece(CellComponent piece) {
		remove(piece);
		for(MouseListener l : getMouseListeners())
			piece.removeMouseListener(l);
	}

	public void removePiece(MapCell piece){
		if(pieces.containsKey(piece))
			removePiece(pieces.get(piece));
	}

	public Point getPieceLocation(MapCell piece){
		if(pieces.containsKey(piece))
			return new Point(pieces.get(piece).x,pieces.get(piece).y);
		else return null;
	}

	public CellComponent[] getComponents(int x,int y,int xOff,int yOff) {
		return layout.getComponents(x,y,xOff,yOff);
	}

	public Point getOrigin(MapLayoutConstraints loc){
		return new Point(loc.point.x*getCellsize().width+loc.offset.x,
				loc.point.y*getCellsize().height+loc.offset.y);
	}

	public MoveThread animateMove(MapCell cell,Point start,Point end,long duration,int steps) {
		CellComponent component=putPiece(start,cell);validate();
		return new MoveThread(cell,new MapLayoutConstraints(start),
				new MapLayoutConstraints(end),duration,steps);
	}

	public MoveThread animateMove(MapCell cell,int x1,int y1,int x2,int y2,long duration,int steps) {
		return animateMove(cell,new Point(x1,y1),new Point(x2,y2),duration,steps);
	}

	public MoveThread animateMove(MapCell cell,int x2,int y2,long duration,int steps){
		CellComponent comp=null;
		if(pieces.containsKey(cell))
			comp=pieces.get(cell);
		else
			comp=putPiece(-1,-1,cell);

		setComponentZOrder(comp, 0);
		return animateMove(cell,comp.x,comp.y,x2,y2,duration,steps);
	}

	public class MoveThread extends Thread{
		MapLayoutConstraints start,end;
		long duration;
		int steps;
		MapCell cell;

		MoveThread(MapCell cell,MapLayoutConstraints start,MapLayoutConstraints end,long duration,int steps){
			this.start=start;
			this.end=end;
			this.cell=cell;
			this.duration=duration;
			this.steps=steps;
		}

		public void run() {
			Point endCoords=getOrigin(end);
			Point startCoords=getOrigin(start);

			int startX,startY,endX,endY;
			startX=start.point.x;
			startY=start.point.y;
			endX=end.point.x;
			endY=end.point.y;

			double cx=start.offset.x;
			double cy=start.offset.y;
			double dx=(double)(endCoords.x-startCoords.x)/(double)(steps);
			double dy=(double)(endCoords.y-startCoords.y)/(double)(steps);

			MapLayoutConstraints cons=cell.getConstraints();
			
			try {				
				for(int n=0;n!=steps;n++)
				{
					cx+=dx;
					cy+=dy;
					cons.offset.x=(int) cx;
					cons.offset.y=(int) cy;
					revalidate();
					sleep(duration/steps);
				}

			}catch(InterruptedException e){}
			finally{
//				cell.setConstraints(end);
				putPiece(endX,endY,cell);
				revalidate();
			}
		}
	}


//	public class MoveThread extends Thread{
//	Point start,end;
//	CellComponent cell;
//	long duration;
//	int steps;

//	MoveThread(CellComponent cell,int x1,int y1,int x2,int y2,long duration,int steps){
//	this(cell,new Point(x1,y1),new Point(x2,y2),duration,steps);
//	}

//	MoveThread(CellComponent cell,Point start,Point end,long duration,int steps){
//	this.start=start;
//	this.end=end;
//	this.cell=cell;
//	this.duration=duration;
//	this.steps=steps;
//	}

//	public void run() {
//	int dx=(end.x-start.x)*getCellsize().width/steps;
//	int dy=(end.y-start.y)*getCellsize().height/steps;

//	MapLayoutConstraints cons=cell.getConstraints();

//	for(int n=0;n!=steps;n++)
//	{
//	cons.translate(dx,dy);
//	cell.setPosition();
//	repaint();

//	try {
//	sleep(duration/steps);
//	}catch(Exception e) {break;}
//	}
//	cons.setOffset(new MapPoint(0,0));
//	cons.setPoint(new MapPoint(end));
//	cell.x=end.x;
//	cell.y=end.y;
//	revalidate();
//	}

//	}
}
