/**
 * 
 */
package org.geekfu.OldCartographer;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager2;
import java.awt.Point;
import java.util.Vector;


class MapLayout implements LayoutManager2{
	/**
	 * 
	 */
	private final Map map;
	Vector<CellComponent> cells;

	public MapLayout(Map map) {
		this.map = map;
		cells=new Vector<CellComponent>();
	}

	public Dimension preferredLayoutSize(Container container) {
		int basex=0,basey=0;
		if(map.getBorder()!=null){
			Insets in=map.getBorder().getBorderInsets(map);
			basex=in.left+in.right;
			basey=in.top+in.bottom;
		}
		return new Dimension(basex+map.getCellsize().width*this.map.width,
				basey+map.getCellsize().height*this.map.height);
	}

	public Dimension minimumLayoutSize(Container container) {
		return preferredLayoutSize(container);
	}

	public void addLayoutComponent(Component component, Object constraints) {
		MapLayoutConstraints c;

		if(!(component instanceof CellComponent))throw new IllegalArgumentException();

		if(constraints instanceof MapPoint)
			c=new MapLayoutConstraints((MapPoint)constraints);
		else if(constraints instanceof MapLayoutConstraints)
			c=(MapLayoutConstraints) constraints;
		else throw new IllegalArgumentException("Have to have either a MapPoint or a MapLayoutConstraints to add a cell");

		((CellComponent)component).setConstraints(c);

		cells.add((CellComponent)component);
	}

	public Dimension maximumLayoutSize(Container container) {
		return preferredLayoutSize(container);
	}

	public float getLayoutAlignmentX(Container arg0) {
		return 0;
	}

	public float getLayoutAlignmentY(Container arg0) {
		return 0;
	}

	public void invalidateLayout(Container arg0) {}


	public void addLayoutComponent(String arg0, Component arg1) {}

	public void removeLayoutComponent(Component component) {
		for(int n=0;n!=cells.size();n++)
			if(cells.get(n)==component)
				cells.remove(n--);
	}

	public CellComponent[] getComponents(int x,int y,int xOff,int yOff) {
		Vector<CellComponent> matches=new Vector<CellComponent>();
		MapLayoutConstraints cons=new MapLayoutConstraints(x,y,xOff,yOff);
		for(int n=0;n!=cells.size();n++)
		{
			CellComponent cell=(CellComponent) cells.get(n);
			if(cons.equals(cell.getConstraints()))
				matches.add(cell);
		}

		CellComponent[] components=new CellComponent[matches.size()];
		for(int n=0;n!=matches.size();n++)
			components[n]=(CellComponent) matches.get(n);

		return components;
	}

	public void layoutContainer(Container arg0) {
		int basex=0,basey=0;
		if(map.getBorder()!=null){
			basex=map.getBorder().getBorderInsets(map).left;
			basey=map.getBorder().getBorderInsets(map).top;
		}
		for(int n=0;n!=cells.size();n++)
		{
			MapLayoutConstraints current=((CellComponent) cells.get(n)).getConstraints();
			Component component=(Component) cells.get(n);

			Point loc=new Point(basex+current.point.x*map.getCellsize().width+current.offset.x,
					basey+current.point.y*map.getCellsize().height+current.offset.y);

			component.setBounds(loc.x,loc.y,map.getCellsize().width,map.getCellsize().height);
		}
	}

}